<?php

//  Copyright (C) 2002-2006 Balint Kis (balint@k-i-s.net)
//  Copyright (C) 2005-2007 Solomon Peachy (pizza@shaftnet.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

include_once "barcode.php";

function ps_DrawCode128Digit($file, $x, $y1, $y2, $n) {
  global $code128Bits;

  if ($n > 107)
    return;

  for($i = 0; $i < 11; $i++) {
    if ($code128Bits[$n][1][$i])
      fwrite($file, "$x $y1 moveto $x $y2 lineto stroke\r\n");
    $x++;
  }
  /* finish sequence */
  if ($n == 106) {
    fwrite($file, "$x $y1 moveto $x $y2 lineto stroke\r\n");
    $x++;
    fwrite($file, "$x $y1 moveto $x $y2 lineto stroke\r\n");
  }
}

function ps_DrawCode128BarCode($file, $string, $type, $storage_info, $xoffset, $yoffset, $width, $height, $border) {
  global $code128Bits;

  $xoffset += $border;
  $yoffset += $border;
  $width   -= $border;
  $height  -= $border;

  $fontsize = $width/15;

  fwrite($file, "gsave\r\n");
  fwrite($file, "0 setgray\r\n");
  fwrite($file, "1 setlinewidth\r\n");
  fwrite($file, "newpath\r\n");

  if ($storage_info != "") {
    $y = $yoffset - $height;
    $x = $xoffset + $width;
    fwrite($file, "$fontsize RF $x $y moveto ($storage_info) RS\r\n");
  }

  switch (type) {
  case 0:
  case 1:
    $ypos = $yoffset - $height;
    fwrite($file, "$fontsize RF $xoffset $ypos moveto ($string) show\r\n");
    break;
  case 2:
    $xpos = $xoffset - $height;
    $ypos = $yoffset - $height;
    fwrite($file, "$fontsize RF $xpos $ypos moveto (");
    for ($i = 0; $i < $stringlength; $i++)
      fwrite($file, "string[$i] ");
    fwrite($file, ") show\r\n");
    break;
  default:
    return;
  }

  fwrite($file, "closepath\r\n");
  fwrite($file, "grestore\r\n");

  $stringlength = strlen($string);
  $pixelwidth = ($stringlength + 3) * 11 + 1;

  if ($width == -$border)
    $xscale = 1;
  else {
    $xscale = $width/$pixelwidth;
    $xoffset *= 1/$xscale;
  }

  if ($height == 0) {
    $height = 30;
  }

  $x = $xoffset;

  fwrite($file, "gsave\r\n");
  fwrite($file, "$xscale 1 scale\r\n");
  fwrite($file, "0 setgray\r\n");
  fwrite($file, "1 setlinewidth\r\n");
  fwrite($file, "newpath\r\n");

  ps_DrawCode128Digit($file, $x, $yoffset - $height + $fontsize, $yoffset, 103+$type);
  $checksum = 103 + $type;
  $x += 11;

  for ($i = 0; $i < $stringlength; $i++) {
    for($j = 0; $j < 107; $j++) {
      if ($code128Bits[$j][0][$type] == $string[$i]) {
        $checksum += $j*($i+1);
        ps_DrawCode128Digit($file, $x, $yoffset - $height + $fontsize, $yoffset, $j);
      }
    }
    $x += 11;
  }

  $checksum = $checksum%103;

  ps_DrawCode128Digit($file, $x, $yoffset - $height + $fontsize, $yoffset, $checksum);
  $x += 11;

  ps_DrawCode128Digit($file, $x, $yoffset - $height + $fontsize, $yoffset, 106);

  fwrite($file, "closepath\r\n");
  fwrite($file, "grestore\r\n");
}


function ps_Header($file, $title, $creator, $username, $version, $width, $height) {
  $date_of_creation = date("g:i a l, F j, Y");
  fwrite($file, "%!PS-Adobe-3.0\r\n");
  fwrite($file, "%%LanguageLevel: 2\r\n");
  fwrite($file, "%%Title: ($title)\r\n");
  fwrite($file, "%%Creator: ($creator)\r\n");
  fwrite($file, "%%CreationDate: ($date_of_creation)\r\n");
  fwrite($file, "%%For: ($username)\r\n");
  fwrite($file, "%%Version: $version\r\n");
  fwrite($file, "%%Pages: 4\r\n");
  fwrite($file, "%%PageOrder: Ascend\r\n");
  fwrite($file, "%%BoundingBox: 0 0 $width $height\r\n");
  fwrite($file, "%%DocumentFonts: Courier-Bold\r\n");
  fwrite($file, "%%DocumentPaperSizes: A4\r\n");
  fwrite($file, "%%EndComments\r\n");
  fwrite($file, "/USERdict 250 dict def\r\n");
}

function ps_PageSetup($file, $format, $width, $height, $orientation) {
  fwrite($file, "%%BeginSetup\r\n");
  fwrite($file, "%%BeginFeature: *Duplex DuplexNoTumble\r\n<<\r\n  /Duplex true\r\n  /Tumble false\r\n>> setpagedevice\r\n%%EndFeature\r\n");
  fwrite($file, "%%BeginFeature: *PageSize $format\r\n<<\r\n  /PageSize [$width $height]\r\n  /ImagingBBox null\r\n  /Orientation $orientation\r\n>> setpagedevice\r\n%%EndFeature\r\n");
  fwrite($file, "%%EndSetup\r\n");
}

function ps_PageBreak($file, $page, $page_display) {
  if ($page != 1) {
    fwrite($file, "showpage\r\n");
  }
  fwrite($file, "%%Page: $page $page_display\r\n");
}

function ps_Footer($file) {
  fwrite($file, "showpage\r\n");
  fwrite($file, "%%EOF\r\n");
}

function ps_Dictionary($file) {
  /* fonts */
  fwrite($file, "/RF {\r\n /size exch def /Arial        findfont [size 0.92 mul 0 0 size 0 0] makefont setfont\r\n} def\r\n");
  fwrite($file, "/CF {\r\n /size exch def /Courier-Bold findfont [size 0.92 mul 0 0 size 0 0] makefont setfont\r\n} def\r\n");
  fwrite($file, "/BF {\r\n /Courier-BoldOblique         findfont [size 1.05 mul 0 0 size 0 0] makefont setfont\r\n} def\r\n");
  fwrite($file, "/CB {\r\n /Courier-Bold                findfont [size 1.05 mul 0 0 size 0 0] makefont setfont\r\n} def\r\n");
  /* right justified text */
  fwrite($file, "/RS {\r\n/txt exch def\r\n txt stringwidth\r\n pop neg 0 rmoveto\r\n txt show\r\n} def\r\n");
  /* boxes */
  fwrite($file, "/Box      {\r\n currentpoint /y exch def /x exch def\r\n 0 setlinewidth\r\n stroke\r\nnewpath\r\n x y 0.05 size mul add moveto\r\n x 0.1 size mul add y 0.05 size mul add 0.1 size mul 180 270 arc\r\n x 0.9 size mul add y 0.05 size mul add 0.1 size mul 270 360 arc\r\n x 0.9 size mul add y 0.65 size mul add 0.1 size mul   0  90 arc\r\n x 0.1 size mul add y 0.65 size mul add 0.1 size mul  90 180 arc\r\n closepath\r\n} def\r\n");
  fwrite($file, "/FlashBox {\r\n currentpoint /y exch def /x exch def\r\n 0 setlinewidth\r\n stroke\r\nnewpath\r\n x y 0.05 size mul add moveto\r\n x 0.1 size mul add y 0.05 size mul add 0.1 size mul 180 270 arc\r\n x 0.9 size mul add y 0.05 size mul add 0.1 size mul 270 360 arc\r\n x 0.9 size mul add y 0.65 size mul add 0.1 size mul   0  90 arc\r\n x 0.1 size mul add y 0.65 size mul add 0.1 size mul  90 180 arc\r\n 0 setlinewidth\r\n stroke\r\n x 0.1 size mul add y 0.60 size mul add moveto\r\n -0.10 size mul -0.25 size mul rlineto\r\n 0.10 size mul 0 rlineto\r\n -0.10 size mul -0.25 size mul rlineto\r\n 0.06 size mul setlinewidth\r\n stroke\r\n size 0.8 mul BF\r\n} def\r\n");
  /* metering modes */
  fwrite($file, "/Metering_Mode_Spot   {\r\n Box\r\n 0 setlinewidth\r\n stroke\r\n x 0.5 size mul add y 0.35 size mul add 0.135 size mul 0 360 arc\r\n fill x 1.1 size mul add y moveto\r\n} def\r\n");
  fwrite($file, "/Metering_Mode_Center {\r\n Box\r\n 0 setlinewidth\r\n stroke\r\n x 0.5 size mul add y 0.35 size mul add 0.12 size mul 0 360 arc fill\r\n 0.04 size mul setlinewidth\r\n x 0.5 size mul add y 0.35 size mul add 0.22 size mul -80 80 arc\r\n stroke\r\n x 0.5 size mul add y 0.35 size mul add 0.22 size mul 100 260 arc\r\n stroke\r\n x 1.1 size mul add y moveto\r\n} def\r\n");
  fwrite($file, "/Metering_Mode_Matrix {\r\n Box fill\r\n 1 setgray\r\n 0.1 size mul setlinewidth x y 0.35 size mul add moveto\r\n size 0 rlineto\r\n x 0.5 size mul add y -0.05 size mul add moveto\r\n 0 0.8 size mul rlineto\r\n stroke\r\n x 0.5 size mul add y 0.35 size mul add 0.2 size mul 0 360 arc fill\r\n 0 setgray\r\n x 0.5 size mul add y 0.35 size mul add 0.12 size mul 0 360 arc fill\r\n x 1.1 size mul add y moveto\r\n} def\r\n");
  /* program modes */
  fwrite($file, "/Mode_Program {\r\ngsave\r\n Box\r\n 0 setlinewidth\r\n stroke\r\n size 0.8 mul BF x 0.13 size mul add y 0.07 size mul add moveto\r\n (P)  show\r\n grestore\r\n x 1.1 size mul add y moveto\r\n} def\r\n");
  fwrite($file, "/Mode_Aperture {\r\ngsave\r\n Box\r\n 0 setlinewidth\r\n stroke\r\n size 0.8 mul BF x 0.16 size mul add y 0.07 size mul add moveto\r\n (A)  show\r\n grestore\r\n x 1.1 size mul add y moveto\r\n} def\r\n");
  fwrite($file, "/Mode_Shutter {\r\ngsave\r\n Box\r\n 0 setlinewidth\r\n stroke\r\n size 0.8 mul BF x 0.13 size mul add y 0.07 size mul add moveto\r\n (S)  show\r\n grestore\r\n x 1.1 size mul add y moveto\r\n} def\r\n");
  fwrite($file, "/Mode_Manual {\r\ngsave\r\n Box\r\n 0 setlinewidth\r\n stroke\r\n size 0.8 mul BF x 0.13 size mul add y 0.07 size mul add moveto\r\n (M)  show\r\n grestore\r\n x 1.1 size mul add y moveto\r\n} def\r\n");
  /* compensations */
  fwrite($file, "/Compensation {\r\ncurrentpoint /y exch def /x exch def\r\nnewpath\r\nx y moveto\r\n0 0.8 size mul rlineto\r\n0.8 size mul 0 rlineto\r\n0 -0.8 size mul rlineto\r\n-0.8 size mul 0 rlineto\r\nclosepath fill\r\n0.08 size mul setlinewidth\r\n1 setgray\r\nx y moveto\r\n0.8 size mul 0.8 size mul rlineto\r\nstroke\r\nx 0.05 size mul add y 0.55 size mul add moveto\r\n0.34 size mul 0 rlineto\r\nstroke\r\nx 0.22 size mul add y 0.38 size mul add moveto\r\n0 0.35 size mul rlineto\r\nstroke\r\nx 0.4 size mul add y 0.2 size mul add moveto\r\n0.34 size mul 0 rlineto\r\nstroke\r\n0 setgray\r\nx 0.9 size mul add y moveto\r\n} def\r\n");
  fwrite($file, "/Flash_Compensation {\r\ngsave\r\nFlashBox\r\n/ox x def /oy y def x\r\n0.20 size mul add y translate\r\n0.85 0.85 scale\r\n0 0 moveto\r\nCompensation\r\ngrestore\r\n/x ox def /y oy def x 1.1 size mul add y moveto\r\n} def\r\n");
  fwrite($file, "/Exposure_Compensation {\r\ngsave\r\nBox\r\n/ox x def /oy y def x\r\n0 setlinewidth\r\nstroke\r\n0.15 size mul add y translate\r\n0.85 0.85 scale\r\n0 0 moveto\r\nCompensation\r\ngrestore\r\n/x ox def /y oy def x 1.1 size mul add y moveto\r\n} def\r\n");
  /* flash modes */
  fwrite($file, "/Flash_Mode_Normal {\r\ngsave\r\nFlashBox\r\nx 0.15 size mul add y 0.07 size mul add moveto (N) show\r\ngrestore\r\nx 1.1 size mul add y moveto\r\n} def\r\n");
  fwrite($file, "/Flash_Mode_Slow {\r\ngsave\r\nFlashBox\r\nx 0.15 size mul add y 0.07 size mul add moveto (S) show\r\ngrestore\r\nx 1.1 size mul add y moveto\r\n} def\r\n");
  fwrite($file, "/Flash_Mode_Rear {\r\ngsave\r\nFlashBox\r\nx 0.15 size mul add y 0.07 size mul add moveto (R) show\r\ngrestore\r\nx 1.1 size mul add y moveto\r\n} def\r\n");
  /* miscellaneous */
  fwrite($file, "/ISO_Code {\r\nBox\r\n0 setlinewidth stroke\r\nnewpath\r\nx 0.1 size mul add y 0.05 size mul add moveto\r\n0 0.5 size mul rlineto\r\n0.3 size mul 0 rlineto\r\n0 -0.5 size mul rlineto\r\n-0.3 size mul 0 rlineto\r\n0.7 setgray\r\nclosepath fill stroke\r\nnewpath\r\nx 0.1 size mul add y 0.05 size mul add moveto\r\n0 0.5 size mul rlineto\r\n0.3 size mul 0 rlineto\r\n0 -0.5 size mul rlineto\r\n-0.3 size mul 0 rlineto\r\n0 setgray\r\nclosepath stroke\r\nnewpath\r\nx 0.2 size mul add y 0.55 size mul add moveto\r\n0 0.1 size mul rlineto\r\n0.1 size mul 0 rlineto\r\n0 -0.1 size mul rlineto\r\n-0.1 size mul 0 rlineto\r\nclosepath stroke\r\nnewpath\r\nx 0.4 size mul add y 0.50 size mul add moveto\r\n0.2 size mul 0 rlineto\r\n0.2 size mul -0.2 size mul rlineto\r\n0.1 size mul 0 rlineto\r\n0 -0.20 size mul rlineto\r\n-0.5 size mul 0 rlineto\r\nclosepath fill stroke\r\nnewpath\r\nx 0.46 size mul add y 0.40 size mul add moveto\r\nx 0.46 size mul add y 0.45 size mul add lineto\r\nx 0.53 size mul add y 0.40 size mul add moveto\r\nx 0.53 size mul add y 0.45 size mul add lineto\r\nx 0.60 size mul add y 0.40 size mul add moveto\r\nx 0.60 size mul add y 0.45 size mul add lineto\r\nx 0.46 size mul add y 0.20 size mul add moveto\r\nx 0.46 size mul add y 0.15 size mul add lineto\r\nx 0.53 size mul add y 0.20 size mul add moveto\r\nx 0.53 size mul add y 0.15 size mul add lineto\r\nx 0.60 size mul add y 0.20 size mul add moveto\r\nx 0.60 size mul add y 0.15 size mul add lineto\r\nx 0.67 size mul add y 0.20 size mul add moveto\r\nx 0.67 size mul add y 0.15 size mul add lineto\r\nx 0.74 size mul add y 0.20 size mul add moveto\r\nx 0.74 size mul add y 0.15 size mul add lineto\r\nx 0.81 size mul add y 0.20 size mul add moveto\r\nx 0.81 size mul add y 0.15 size mul add lineto\r\n1 setgray\r\n0.03 size mul setlinewidth closepath stroke\r\n0 setgray\r\nx 1.1 size mul add y moveto\r\n} def\r\n");
  fwrite($file, "/Focal_Length {\r\nBox\r\n0 setlinewidth\r\nstroke\r\nnewpath\r\nx 0.1 size mul add y 0.05 size mul add moveto\r\n0 0.6 size mul rlineto\r\n0.25 size mul 0 rlineto\r\n0 -0.6 size mul rlineto\r\n-0.25 size mul 0 rlineto\r\n0 size mul setlinewidth\r\nclosepath stroke\r\nnewpath\r\nx 0.35 size mul add y 0.10 size mul add moveto\r\n0 0.50 size mul rlineto\r\n0.2 size mul 0 rlineto\r\n0 -0.50 size mul rlineto\r\n-0.2 size mul 0 rlineto\r\n0 size mul setlinewidth\r\n0.7 setgray\r\nclosepath fill stroke\r\nnewpath\r\nx 0.35 size mul add y 0.10 size mul add moveto\r\n0 0.50 size mul rlineto\r\n0.2 size mul 0 rlineto\r\n0 -0.50 size mul rlineto\r\n-0.2 size mul 0 rlineto\r\n0 size mul setlinewidth\r\n0 setgray\r\nclosepath stroke\r\nnewpath\r\nx 0.55 size mul add y 0.10 size mul add moveto\r\n0 0.50 size mul rlineto\r\n0.2 size mul 0 rlineto\r\n0 -0.50 size mul rlineto\r\n-0.2 size mul 0 rlineto\r\n0 size mul setlinewidth\r\nclosepath stroke\r\nnewpath\r\nx 0.75 size mul add y 0.125 size mul add moveto\r\n0 0.45 size mul rlineto\r\n0.10 size mul 0 rlineto\r\n0 -0.45 size mul rlineto\r\n-0.10 size mul 0 rlineto\r\n0 size mul setlinewidth\r\n0.7 setgray\r\nclosepath fill stroke\r\nnewpath\r\nx 0.75 size mul add y 0.125 size mul add moveto\r\n0 0.45 size mul rlineto\r\n0.10 size mul 0 rlineto\r\n0 -0.45 size mul rlineto\r\n-0.10 size mul 0 rlineto\r\n0 size mul setlinewidth\r\n0 setgray\r\nclosepath stroke\r\nnewpath\r\nx 0.85 size mul add y 0.175 size mul add moveto\r\n0 0.35 size mul rlineto\r\n0.04 size mul 0 rlineto\r\n0 -0.35 size mul rlineto\r\n-0.04 size mul 0 rlineto\r\n0 size mul setlinewidth\r\nclosepath stroke\r\nx 1.1 size mul add y moveto\r\n} def\r\n");
  fwrite($file, "/Text_Copyright {\r\n/txt exch def txt stringwidth 0 rmoveto\r\ncurrentpoint\r\n/y exch def /x exch def\r\nnewpath\r\n0 setlinewidth\r\nx 0.4 size mul add y 0.4 size mul add 0.45 size mul 0 360 arc\r\nstroke\r\nx y moveto\r\n0.1 size mul 0.1 size mul rmoveto\r\ngsave\r\n0.9 size mul CF (C) show\r\ngrestore\r\nx 1.1 size mul add y moveto txt show\r\n} def\r\n\r\n");
  fwrite($file, "/WRAP {\r\n/width exch def\r\n/leading_edge exch def\r\n/font_size exch def\r\n/input_string exch def\r\ninput_string stringwidth /wx exch def /wy exch def\r\nwy width lt {\r\ninput_string show\r\n} {\r\n/total_length 0 def\r\n{\r\ninput_string ( )search {\r\n/word exch def pop\r\n/input_string exch def\r\nword stringwidth /sx exch def /sy exch def\r\ntotal_length sy add /total_length exch def\r\ntotal_length width ge {\r\n/total_length sy def\r\ncurrentpoint font_size 1.1 mul sub leading_edge exch moveto\r\n} if\r\nword show\r\n( ) show\r\n( ) stringwidth /sx exch def /sy exch def\r\ntotal_length sy add /total_length exch def\r\n} {\r\n show\r\n exit\r\n} ifelse\r\n} loop\r\n} ifelse\r\n} def\r\n");
}

function ps_FrameInfo($file, $xoffset, $yoffset, $width, $height, $border, $frame_data) {
  $fontsize = $height/5;
  if ($width == 0) $width = 160;
  if ($height == 0) $height = 40;

  $xoffset += $border;
  $yoffset += $border;
  $width   -= $border;
  $height  -= $border;

  $rect_x = $xoffset - 2;
  $rect_y = $yoffset + 2;
  $rect_w = $width + 2;
  $rect_h = - $height - 2;

  $row_printed = 0;

  fwrite($file, "gsave\r\n");
  fwrite($file, "$rect_x $rect_y $rect_w $rect_h rectclip\r\n"); /* rectstroke */
  fwrite($file, "0 setgray\r\n");
  fwrite($file, "newpath\r\n");

  /* copyright note, date */
  $y = $yoffset - $fontsize;
  fwrite($file, "$fontsize RF $xoffset $y moveto ");
  if ($frame_data['copyright'] != "")
    fwrite($file, "(".$frame_data['copyright'].") Text_Copyright\r\n");

  /* frame data: program mode, metering mode ... */
  $y = $y - $fontsize - 1;
  fwrite($file, "$fontsize RF $xoffset $y moveto ");

  /* shutter F aperture */
  if ($frame_data['shutter'] != "") {
    $row_printed = 1;
    fwrite($file, "$fontsize RF (".$frame_data['shutter'].") show ");
  }

  if ($frame_data['aperture'] != "") {
    $row_printed = 1;
    fwrite($file, "$fontsize CB (F) show ");
    fwrite($file, "$fontsize RF (".$frame_data['aperture'].") show ");
  }

  /* exposure compensation */
  if ($frame_data['exp_comp'] != "") {
    $row_printed = 1;
    fwrite($file, "Exposure_Compensation (".$frame_data['exp_comp'].") show ");
  }

  /* camera program mode */
  if ($frame_data['camera_program'] != "") {
    $row_printed = 1;
    switch ($frame_data['camera_program']) {
    default:
    case 1: fwrite($file, "Mode_Program "); break;
    case 2: fwrite($file, "Mode_Aperture "); break;
    case 3: fwrite($file, "Mode_Shutter "); break;
    case 4: fwrite($file, "Mode_Manual "); break;
    }
  }

  /* camera metering mode */
  if ($frame_data['camera_meter'] != "") {
    $row_printed = 1;
    switch ($frame_data['camera_meter']) {
    case 1: fwrite($file, "Metering_Mode_Spot "); break;
    case 2: fwrite($file, "Metering_Mode_Center "); break;
    default:
    case 3: fwrite($file, "Metering_Mode_Matrix "); break;
    }
  }

  /* flash mode */
  if ($frame_data['flash_mode'] != "") {
    $row_printed = 1;
    switch ($frame_data['flash_mode']) {
    default:
    case 1: fwrite($file, "Flash_Mode_Normal "); break;
    case 2: fwrite($file, "Flash_Mode_Slow "); break;
    case 3: fwrite($file, "Flash_Mode_Rear "); break;
    }
  }

  /* flash compensation */
  if ($frame_data['flash_comp'] != "") {
    $row_printed = 1;
    fwrite($file, "Flash_Compensation (".$frame_data['flash_comp'].") show ");
  }

  if ($frame_data['iso'] != "") {
    $row_printed = 1;
    if ($frame_data['iso_override'] != "")
      $frame_data['iso_override'] = "[" . $frame_data['iso_override'] . "]";
    fwrite($file, "ISO_Code (".$frame_data['iso']." ".$frame_data['iso_override'].") show\r\n");
  }

  if ($frame_data['focal_length'] != "") {
    $row_printed = 1;
    fwrite($file, "Focal_Length (".$frame_data['focal_length'].") show\r\n");
  }

  if ($frame_data['camera'] != "") {
    $row_printed = 1;
    fwrite($file, "$fontsize RF ( ".$frame_data['camera'].") show\r\n");
  }

  if ($row_printed == 1) {
    $y = $y - $fontsize - 1;
    fwrite($file, "$fontsize RF $xoffset $y moveto ");
  }

  $row_printed = 0;
  if ($frame_data['title'] != "") {
    $row_printed = 1;
    fwrite($file, "(".$frame_data['title'].") show\r\n");
  }

  if ($row_printed == 1) {
    $y = $y - $fontsize - 1;
    fwrite($file, "$fontsize RF $xoffset $y moveto ");
  }

  $row_printed = 0;
  if ($frame_data['location'] != "") {
    fwrite($file, "(".$frame_data['location'].") $fontsize $xoffset $rect_w WRAP\r\n");
    $row_printed = 1;
  }

  if ($row_printed == 1) {
    $y = $y - $fontsize - 1;
    fwrite($file, "$fontsize RF $xoffset $y moveto ");
  }

  $row_printed = 0;
  if ($frame_data['caption'] != "") {
    fwrite($file, "(".$frame_data['caption'].") $fontsize $xoffset $rect_w WRAP\r\n");
    $row_printed = 1;
  }

  fwrite($file, "closepath\r\n");
  fwrite($file, "grestore\r\n");

  return $label_area;
}

function ps_ScaleImage($image_file, $xoffset, $yoffset, $width, $height, $border) {
  $xoffset += $border;
  $yoffset += $border;
  $width   -= $border;
  $height  -= $border;

  $image_size = getimagesize($image_file);
  $scaled_image = array(0, 0, 0, 0, 0, 0);

  $original_width = $width;
  $original_height = $height;

  if ($image_size[0] > $image_size[1])
    $height = 0;
  else
    $width = 0;

  if ($width == -$border && $height != -$border) {
    $scaled_image[5] = $height/$image_size[1];
    $scaled_image[4] = $scaled_image[5];
    $scaled_image[2] = $image_size[0]*$scaled_image[5];
    $scaled_image[3] = $height;
  }
  elseif ($height == -$border && $width != -$border) {
    $scaled_image[4] = $width/$image_size[0];
    $scaled_image[5] = $scaled_image[4];
    $scaled_image[2] = $width;
    $scaled_image[3] = $image_size[1]*$scaled_image[4];
  }
  else {
    $scaled_image[4] = 1;
    $scaled_image[5] = 1;
    $scaled_image[2] = $image_size[0];
    $scaled_image[3] = $image_size[1];
  }

  $scaled_image[0] = $xoffset;
  $scaled_image[1] = $yoffset - $scaled_image[3];

  if ($scaled_image[3] < $scaled_image[2]) {
    $scaled_image[1] -= ($original_height - $scaled_image[3]) / 2;
    $scaled_image[3] = $scaled_image[2];
  }
  else {
    $scaled_image[0] += ($original_width - $scaled_image[2]) / 2;
    $scaled_image[2] = $scaled_image[3];
  }

  return $scaled_image;
}


function ps_PrintJPEG($file, $image_file, $xoffset, $yoffset, $xscale, $yscale) {
  $file_size = filesize($image_file);
  $image_size = getimagesize($image_file);
  $fd = fopen ($image_file, "rb");
  $file_contents = bin2hex(fread($fd, $file_size));
  fclose($fd);

  fwrite($file, "gsave\r\n");
  fwrite($file, "newpath\r\n");
  fwrite($file, "$xoffset $yoffset translate\r\n");
  fwrite($file, "$xscale $image_size[0] mul $yscale $image_size[1] mul scale\r\n");
  fwrite($file, "/DeviceRGB setcolorspace\r\n");
  fwrite($file, "<< /ImageType 1\r\n");
  fwrite($file, "   /Width $image_size[0]\r\n");
  fwrite($file, "   /Height $image_size[1]\r\n");
  fwrite($file, "   /ImageMatrix [$image_size[0] 0 0 -$image_size[1] 0 $image_size[1]]\r\n");
  fwrite($file, "   /DataSource currentfile /ASCIIHexDecode filter /DCTDecode filter\r\n");
  fwrite($file, "   /BitsPerComponent 8\r\n");
  fwrite($file, "   /Decode [0 1 0 1 0 1]\r\n");
  fwrite($file, ">> image ");
  fwrite($file, $file_contents);
  fwrite($file, " >\r\n");
  fwrite($file, "closepath stroke\r\n");
  fwrite($file, "grestore\r\n");

  fwrite($file, "gsave\r\n");
  fwrite($file, "newpath\r\n");
  fwrite($file, "$xoffset $yoffset translate\r\n");
  fwrite($file, "$xscale $yscale scale\r\n");
  fwrite($file, "0 setgray\r\n");
  fwrite($file, "1 setlinewidth stroke\r\n");
  fwrite($file, "0 0 moveto\r\n");
  fwrite($file, "0 $image_size[1] 0 add rlineto\r\n");
  fwrite($file, "$image_size[0] 0 add 0 rlineto\r\n");
  fwrite($file, "0 $image_size[1] neg 0 add rlineto\r\n");
  fwrite($file, "$image_size[0] neg 0 add 0 rlineto\r\n");
  fwrite($file, "closepath stroke\r\n");
  fwrite($file, "grestore\r\n");
}

function ps_PrintCross($file, $x, $y, $size) {
  return;

  fwrite($file, "gsave\r\n");
  fwrite($file, "1 1 scale\r\n");
  fwrite($file, "0 setgray\r\n");
  fwrite($file, "0 setlinewidth\r\n");
  fwrite($file, "newpath\r\n");

  $x1 = $x;
  $y1 = $y - $size;
  $x2 = $x;
  $y2 = $y + $size;
  fwrite($file, "$x1 $y1 moveto $x2 $y2 lineto stroke\r\n");
  $x1 = $x - $size;
  $y1 = $y;
  $x2 = $x + $size;
  $y2 = $y;
  fwrite($file, "$x1 $y1 moveto $x2 $y2 lineto stroke\r\n");

  fwrite($file, "closepath\r\n");
  fwrite($file, "grestore\r\n");
}

?>
