
--  Copyright (C) 2002, 2003, 2004, 2005, 2006 Balint Kis (balint@k-i-s.net)

--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 2 of the License, or
--  (at your option) any later version.

--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.

--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

----------------------------------------------------------
-- v2.28 to v2.29 patch
----------------------------------------------------------

alter table album add column  parent_album integer references album(identifier);

alter table camera_type add raw_icc_profile integer;

drop view view_camera_type;
create view view_camera_type as
  select camera_type.identifier, manufacturer.name, model, variation,
    manufacturer.url, raw_icc_profile
    from camera_type, manufacturer
   where camera_type.manufacturer = manufacturer.identifier
   order by manufacturer.name, model, variation;

drop view view_camera;
create view view_camera as
  select camera.identifier, manufacturer.name, model, variation,
camera.serial_number, camera.users, date_of_purchase,
camera.purchased_new,
manufacturer.url, (access_type.value) as access_rights, raw_icc_profile
    from camera_type, camera, manufacturer, access_type
   where camera_type.manufacturer = manufacturer.identifier
     and camera_type.identifier = camera.type
     and camera.access_rights = access_type.identifier
   order by manufacturer.name, model, variation;

alter table imagemagick_options add preview_sharpen_threshold float;
alter table imagemagick_options add preview_sharpen_amount float;
alter table imagemagick_options add thumb_sharpen_threshold float;
alter table imagemagick_options add thumb_sharpen_amount float;

-- begin: drop access constraint on the bulk_update_rule
create table bulk_update_rule_temp as select * from bulk_update_rule;

drop table bulk_update_rule;

create table bulk_update_rule (
  identifier             integer not null primary key,
  users                  integer not null references users(identifier),
  rule_name              varchar(300),
  regenerate_thumbnail   char(1) check (regenerate_thumbnail in ('t', 'f')),
  regenerate_preview     char(1) check (regenerate_preview in ('t', 'f')),
  title                  text,
  author                 text,
  keyword                text,
  headline               text,
  caption_writer         text,
  caption                text,
  category               varchar(3),
  supplemental_category  text,
  credit                 text,
  copyright_statement    text,
  web_statement          text,
  instructions           text,
  source                 text,
  transmission_reference text,
  date_of_exposure       timestamp,
  access_rights          integer,
  hide_original          char(1) check (hide_original in ('t', 'f')),
  location               integer references location(identifier),
  camera                 integer references camera(identifier),
  camera_metering        integer references camera_metering(identifier),
  camera_program         integer references camera_program(identifier),
  lens                   integer references lens(identifier),
  focal_length           integer references focal_length(identifier),
  filter                 integer references filter(identifier),
  film                   integer references film(identifier),
  iso_override           integer references film_iso(identifier),
  aperture               integer references aperture(identifier),
  shutter                integer references shutter(identifier),
  exposure_comp          integer references ev_comp(identifier),
  ev_difference          integer references ev_comp(identifier),
  flash                  integer references flash(identifier),
  flash_mode             integer references flash_mode(identifier),
  flash_comp             integer references ev_comp(identifier),
  scanner                integer references scanner(identifier),
  scan_resolution        integer references scan_resolution(identifier),
  scan_bitdepth          integer references scan_bitdepth(identifier),
  scan_multiscan         integer references scan_multiscan(identifier),
  support                integer references support(identifier)
);


insert into bulk_update_rule select * from bulk_update_rule_temp;
drop table bulk_update_rule_temp;
-- end: drop users constraint on the submissions

-- create a special Orphanage for deleted photos that were involved in some kind of transaction
insert into folder (identifier, parent_folder, users, caption, date_of_creation, access_rights, orderby, description)
  select nextval('folder_id_sequence') as identifier,
         null as parent_folder,
         identifier as users,
         'Orphanage' as caption,
         now() as date_of_creation,
         3 as access_rights,
         7 as orderby,
         null as description
    from users;

create function photo_shop_transaction_references(integer) returns bigint
as 'select count(shop_order.transaction)
      from photo_item, shop_order
     where photo_item.photo = $1
       and photo_item.identifier=shop_order.item' language 'sql';

alter table photo_item add deleted char(1) check (deleted in ('t', 'f'));
